/*************************************************************************
 *	  @file Test.java
 *
 *	  @brief Tests a SSL/TLS connection to the Apriva server for ReadyTouch
 *
 *	  @author Robert Smith, Patty Heit
 *
 *	  @date 05/15/2012
 *
 *	  Copyright (c) 2013 by Apriva
 *	  All rights reserved
 *
 *	  CONFIDENTIAL AND PROPRIETARY
 *
 *	  This software may contain confidential and trade secret
 *	  information and technology and may not be used, disclosed or
 *	  made available to others without the permission of Apriva.
 *	  Copies may only be made with permission and must contain the
 *	  above copyright notice. Neither title to the software nor
 *	  ownership of the software is hereby transferred.
 *
 *	  $Id$
 *************************************************************************/
import java.io.*;
import java.security.*;
import javax.net.ssl.*;


/////////////////////////////////////////////////////
// A test program for ReadyTouch to connect to the Apriva server using TLS
/////////////////////////////////////////////////////
class Test {

	/////////////////////////////////////////////////////
	// Create an SSL socket factory to use to connect to the Apriva server with
	// Read the appropriate certificate chains and keys from files into the SSL factory
	/////////////////////////////////////////////////////
	protected static javax.net.ssl.SSLSocketFactory createSSLFactory () {

		try {
			// *** Client Side Certificate *** //
			System.out.println ("2. Loading p12 file");

			// Load the certificate file into the keystore
			KeyStore keystore = KeyStore.getInstance("PKCS12");
			FileInputStream inputFile = new FileInputStream (clientCertFileName);

			char [] clientPassphrase = clientCertPassword.toCharArray ();
			keystore.load (inputFile, clientPassphrase);

			// Create the factory
			KeyManagerFactory keyManagerFactory = KeyManagerFactory.getInstance("SunX509");
			keyManagerFactory.init (keystore, clientPassphrase);

			// *** Server Trust *** //
			System.out.println ("3. Loading JKS file");
			KeyStore truststore = KeyStore.getInstance("JKS");
			FileInputStream trustInputFile = new FileInputStream (serverTrustFileName);

			char [] serverTrustPassphrase = serverTrustPassword.toCharArray ();
			truststore.load (trustInputFile, serverTrustPassphrase);

			TrustManagerFactory tmf = TrustManagerFactory.getInstance(TrustManagerFactory.getDefaultAlgorithm());
			tmf.init (truststore);

			TrustManager[] trustManagers = tmf.getTrustManagers ();

			// Create the SSL context and use it to initialize the factory
			SSLContext ctx = SSLContext.getInstance("TLS");
			ctx.init (keyManagerFactory.getKeyManagers(), trustManagers, null);

			SSLSocketFactory sslFactory = ctx.getSocketFactory();
			return sslFactory;

		} catch (Exception e) {

			e.printStackTrace ();
		}

		return null;
	}


	/////////////////////////////////////////////////////
	// Perform the test by connecting to the Apriva server
	/////////////////////////////////////////////////////

	protected static void test (String host, int port) {

		try {
			// Create an SSL factory and use it to create an SSL socket
			SSLSocketFactory sslFactory = createSSLFactory ();

			System.out.println ("4. Connecting to " + host +  " port " + port);
			SSLSocket socket = (SSLSocket) sslFactory.createSocket (host, port);

			// Disable weak algorithms and only enable strong algorithms
			// String[] goodCiphers ={"TLS_DHE_RSA_WITH_AES_128_CBC_SHA", "TLS_DHE_DSS_WITH_AES_128_CBC_SHA", "TLS_RSA_WITH_AES_128_CBC_SHA"};
			// socket.setEnabledCipherSuites (goodCiphers); // Note - the Apriva server seems to reject the handshake when the algorithms are restricted
			// Connect

			socket.startHandshake();

			/////////////////////////////////////////////////////
			// Write some XML here
			/////////////////////////////////////////////////////
			OutputStream outputstream = socket.getOutputStream();
			OutputStreamWriter outputstreamwriter = new OutputStreamWriter(outputstream);

			BufferedWriter bufferedWriter = new BufferedWriter(outputstreamwriter);

			String testXML = "<AprivaPosXml><Credit MessageType=\"Request\" Version=\"5.0\" ProcessingCode=\"Sale\"><ClerkID>ClerkID1</ClerkID><PosConditionCode>1</PosConditionCode><CardPresent>YES</CardPresent><EntryMode>Manual</EntryMode><EntryModeType>Standard</EntryModeType><ExpireDate>12/12</ExpireDate><TransactionDate>12/12</TransactionDate><TransactionTime>01:01:01</TransactionTime><Stan>1</Stan><Amount>1.00</Amount><AccountNumber>4111111111111113</AccountNumber></Credit></AprivaPosXml>";

			System.out.println ("5. Sending Request --->>>>>>");
			System.out.println (testXML);
			
			bufferedWriter.write (testXML);
			bufferedWriter.flush ();

			System.out.println ("6. Waiting for Response <<<<<<--------");
			InputStream inputstream = socket.getInputStream();
			InputStreamReader inputstreamreader = new InputStreamReader(inputstream);
			BufferedReader bufferedReader = new BufferedReader(inputstreamreader);

			String line = null;
			while ((line = bufferedReader.readLine()) != null) {
				System.out.println(line);
			}

		} catch (Exception e) {
			e.printStackTrace ();
		}
	}


	/////////////////////////////////////////////////////
	// Main - just run the test
	/////////////////////////////////////////////////////
	public static void main(String[] args)
	{

		if (args.length != 2)
		{
			System.out.println ("Invalid Parameters");
			System.out.println ("Required Usage : <DstIP> <DstPort>");
			System.out.println ("Optional Usage : <DstIP> <DstPort> <Client p12 filename> <p12 password> <Trust jks filename> <jks password>");
			return;
		}
		
		// Specify the inputs
		// The file containing the client certificate, private key, and chain
		clientCertFileName = "Client.p12";
		clientCertPassword = "P@ssword";//"password";

		// The file containing the server trust chain
		serverTrustFileName = "AprivaTrust.jks";
		serverTrustPassword = "P@ssword";
		
		if (args.length == 4)
		{
			clientCertFileName = args[2];
			clientCertFileName = args[3];
		}
		else if (args.length == 6)
		{
			clientCertFileName = args[2];
			clientCertFileName = args[3];
			serverTrustFileName = args[4];
			serverTrustPassword = args[5];
		}
		else if (args.length > 2)
		{
			System.out.println ("Invalid Parameters");
			System.out.println ("Required Usage : <DstIP> <DstPort>");
			System.out.println ("Optional Usage : <DstIP> <DstPort> <Client p12 filename> <p12 password> <Trust jks filename> <jks password>");
			return;
		}

		String host = args[0];
		int port = Integer.parseInt(args[1]);

		System.out.println ("1. Running Test");
		test (host, port);
	}

	
	static String clientCertFileName;
	static String clientCertPassword;
	static String serverTrustFileName;
	static String serverTrustPassword;
	
}



